/*
 * Video capture module utilising the GStreamer pipeline for AR Toolkit
 * 
 * (c) Copyrights 2003-2006 Hartmut Seichter
 * 
 * Licensed under the terms of the GPL
 *
 */
#ifndef AR_VIDEO_GSTREAMER_H
#define AR_VIDEO_GSTREAMER_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <AR/config.h>
#include <AR/ar.h>

typedef struct _AR2VideoParamT AR2VideoParamT;

#ifdef  __cplusplus
}
#endif

#endif
