 /* ========================================================================
  * PROJECT: DirectShow Video Processing Library
  * FILE:    ARToolkit 2.6x WIN32/DirectShow video input
  * ========================================================================
  * Author:  Thomas Pintaric, Vienna University of Technology
  * Contact: pintaric@ims.tuwien.ac.at http://ims.tuwien.ac.at/~thomas
  * =======================================================================
  *
  * Copyright (C) 2003  Vienna University of Technology
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * For further information please contact Thomas Pintaric under
  * <pintaric@ims.tuwien.ac.at> or write to Thomas Pintaric,
  * Vienna University of Technology, Favoritenstr. 9-11/188, A1040 Vienna,
  * Austria.
  * ========================================================================*/
/*
 *	Copyright (c) 2004-2007 Philip Lamb (PRL) phil@eden.net.nz. All rights reserved.
 *	
 *	Rev		Date		Who		Changes
 *	2.6.8	2004-07-20	PRL		Rewrite for ARToolKit 2.68.2
 *
 */

#ifndef AR_VIDEO_WIN32_DIRECTSHOW_H
#define AR_VIDEO_WIN32_DIRECTSHOW_H

#include <AR/config.h>
#include <AR/ar.h>

#include <stdio.h>
#include <string.h>

#define AR_VIDEO_WIN32_DIRECTSHOW_2_71

#ifdef  __cplusplus
extern "C" {
#endif
	
typedef struct _AR2VideoBufferT AR2VideoBufferT;
typedef struct _AR2VideoParamT AR2VideoParamT;

#ifdef  __cplusplus
}
#endif
#endif // AR_VIDEO_WIN32_DIRECTSHOW_H
