/*
    Psychtoolbox-3/PsychSourceGL/Source/Common/Eyelink/RegisterProject.h

    PROJECTS: Eyelink

    AUTHORS:

    cburns@berkeley.edu             cdb
    E.Peters@ai.rug.nl              emp
    f.w.cornelissen@med.rug.nl      fwc

    PLATFORMS: All

    HISTORY:

    11/21/05  cdb       Created.
*/

// begin include once
#ifndef PSYCH_IS_INCLUDED_RegisterProject
#define PSYCH_IS_INCLUDED_RegisterProject

#define PSYCH_NOLM

#include "PsychEyelink.h"

PsychError PsychModuleInit(void);

// end include once
#endif
