function glEndConditionalRender

% glEndConditionalRender  Interface to OpenGL function glEndConditionalRender
%
% usage:  glEndConditionalRender
%
% C function:  void glEndConditionalRender(void)

% 30-Sep-2014 -- created (generated automatically from header files)

if nargin~=0,
    error('invalid number of arguments');
end

moglcore( 'glEndConditionalRender' );

return
